﻿/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;

namespace BeastLink
{
    /// <summary>
    /// Direct access to the BeastLink-Core-API. Internal use only.
    /// </summary>
    internal static class SystemConnector
    {
        private const string Prefix = "beastlink";
        private const string Version = "1.0";
        private const string DLLx86 = Prefix + "-" + Version + "-x86.dll";
        private const string DLLx64 = Prefix + "-" + Version + "-x86_64.dll";
        private const string DLLNoA = Prefix + "-" + Version + ".dll";

        internal const int BL_SUCCESS = 0;
        internal const int BL_FAILED = -1;
        internal const int BL_INVALID_DEVICE = -1;
        internal const int BL_IF_BUFFER_SIZE = 16384;
        internal const int BL_FLAG_CONST_ADDR = 0x00000001;   // Use const address in block I/O (BlReadBlock/BlWriteBlock).

        private static bool Is64 { get { return IntPtr.Size == 8; } }
        private static bool IsNoArch { get { return Environment.OSVersion.Platform != PlatformID.Win32NT; } }

        private static string PtrToStringUtf8(IntPtr ptr)
        {
            if (ptr == IntPtr.Zero)
                return "";

            int len = 0;
            while (Marshal.ReadByte(ptr, len) != 0)
                len++;

            if (len == 0)
                return "";

            var array = new byte[len];
            Marshal.Copy(ptr, array, 0, len);
            return Encoding.UTF8.GetString(array);
        }

        private static void CheckError(int errorCode)
        {
            if (errorCode != BL_SUCCESS)
            {
                var sb = new StringBuilder(BL_IF_BUFFER_SIZE);
                if (IsNoArch)
                    BlGetLastErrorTextNA(errorCode, sb);
                else if (Is64)
                    BlGetLastErrorText64(errorCode, sb);
                else
                    BlGetLastErrorText86(errorCode, sb);
                throw new IOException(sb.ToString());
            }
        }

        /// Error handling.
        [DllImport(DLLx86, EntryPoint = "BlGetLastErrorText", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern void BlGetLastErrorText86(int id, StringBuilder name);
        [DllImport(DLLx64, EntryPoint = "BlGetLastErrorText", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern void BlGetLastErrorText64(int id, StringBuilder name);
        [DllImport(DLLNoA, EntryPoint = "BlGetLastErrorText", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern void BlGetLastErrorTextNA(int id, StringBuilder name);

        /// Global library functionality.
        [DllImport(DLLx86, EntryPoint = "BlGetVersion", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern void BlGetVersion86(ref int major, ref int minor, ref int fix);
        [DllImport(DLLx64, EntryPoint = "BlGetVersion", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern void BlGetVersion64(ref int major, ref int minor, ref int fix);
        [DllImport(DLLNoA, EntryPoint = "BlGetVersion", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern void BlGetVersionNA(ref int major, ref int minor, ref int fix);
        internal static void BlGetVersion(ref int major, ref int minor, ref int fix)
        {
            if (IsNoArch)
                BlGetVersionNA(ref major, ref minor, ref fix);
            else if (Is64)
                BlGetVersion64(ref major, ref minor, ref fix);
            else
                BlGetVersion86(ref major, ref minor, ref fix);
        }

        [DllImport(DLLx86, EntryPoint = "BlInit", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlInit86();
        [DllImport(DLLx64, EntryPoint = "BlInit", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlInit64();
        [DllImport(DLLNoA, EntryPoint = "BlInit", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlInitNA();
        internal static void BlInit()
        {
            if (IsNoArch)
                CheckError(BlInitNA());
            else if (Is64)
                CheckError(BlInit64());
            else
                CheckError(BlInit86());
        }

        [DllImport(DLLx86, EntryPoint = "BlEnumerate", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlEnumerate86(int vid, int pid, ref int count);
        [DllImport(DLLx64, EntryPoint = "BlEnumerate", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlEnumerate64(int vid, int pid, ref int count);
        [DllImport(DLLNoA, EntryPoint = "BlEnumerate", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlEnumerateNA(int vid, int pid, ref int count);
        internal static int BlEnumerate(int vid, int pid)
        {
            int r = 0;
            if (IsNoArch)
                CheckError(BlEnumerateNA(vid, pid, ref r));
            else if (Is64)
                CheckError(BlEnumerate64(vid, pid, ref r));
            else
                CheckError(BlEnumerate86(vid, pid, ref r));
            return r;
        }

        [DllImport(DLLx86, EntryPoint = "BlCleanup", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlCleanup86();
        [DllImport(DLLx64, EntryPoint = "BlCleanup", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlCleanup64();
        [DllImport(DLLNoA, EntryPoint = "BlCleanup", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlCleanupNA();
        internal static void BlCleanup()
        {
            if (IsNoArch)
                CheckError(BlCleanupNA());
            else if (Is64)
                CheckError(BlCleanup64());
            else
                CheckError(BlCleanup86());
        }

        [DllImport(DLLx86, EntryPoint = "BlSetLogLevel", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetLogLevel86(int level);
        [DllImport(DLLx64, EntryPoint = "BlSetLogLevel", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetLogLevel64(int level);
        [DllImport(DLLNoA, EntryPoint = "BlSetLogLevel", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetLogLevelNA(int level);
        internal static void BlSetLogLevel(int level)
        {
            if (IsNoArch)
                CheckError(BlSetLogLevelNA(level));
            else if (Is64)
                CheckError(BlSetLogLevel64(level));
            else
                CheckError(BlSetLogLevel86(level));
        }

        /// Device access.
        [DllImport(DLLx86, EntryPoint = "BlOpen", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlOpen86(int enumId, ref int handle);
        [DllImport(DLLx64, EntryPoint = "BlOpen", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlOpen64(int enumId, ref int handle);
        [DllImport(DLLNoA, EntryPoint = "BlOpen", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlOpenNA(int enumId, ref int handle);
        internal static int BlOpen(int enumId)
        {
            int r = 0;
            if (IsNoArch)
                CheckError(BlOpenNA(enumId, ref r));
            else if (Is64)
                CheckError(BlOpen64(enumId, ref r));
            else
                CheckError(BlOpen86(enumId, ref r));
            return r;
        }

        [DllImport(DLLx86, EntryPoint = "BlClose", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlClose86(int handle);
        [DllImport(DLLx64, EntryPoint = "BlClose", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlClose64(int handle);
        [DllImport(DLLNoA, EntryPoint = "BlClose", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlCloseNA(int handle);
        internal static void BlClose(int handle)
        {
            if (IsNoArch)
                CheckError(BlCloseNA(handle));
            else if (Is64)
                CheckError(BlClose64(handle));
            else
                CheckError(BlClose86(handle));
        }

        /// I/O functionality.
        [DllImport(DLLx86, EntryPoint = "BlReadRegister", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlReadRegister86(int handle, uint reg, ref uint result);
        [DllImport(DLLx64, EntryPoint = "BlReadRegister", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlReadRegister64(int handle, uint reg, ref uint result);
        [DllImport(DLLNoA, EntryPoint = "BlReadRegister", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlReadRegisterNA(int handle, uint reg, ref uint result);
        internal static uint BlReadRegister(int handle, uint reg)
        {
            uint r = 0;
            if (IsNoArch)
                CheckError(BlReadRegisterNA(handle, reg, ref r));
            else if (Is64)
                CheckError(BlReadRegister64(handle, reg, ref r));
            else
                CheckError(BlReadRegister86(handle, reg, ref r));
            return r;
        }

        [DllImport(DLLx86, EntryPoint = "BlWriteRegister", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlWriteRegister86(int handle, uint reg, uint value);
        [DllImport(DLLx64, EntryPoint = "BlWriteRegister", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlWriteRegister64(int handle, uint reg, uint value);
        [DllImport(DLLNoA, EntryPoint = "BlWriteRegister", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlWriteRegisterNA(int handle, uint reg, uint value);
        internal static void BlWriteRegister(int handle, uint reg, uint value)
        {
            if (IsNoArch)
                CheckError(BlWriteRegisterNA(handle, reg, value));
            else if (Is64)
                CheckError(BlWriteRegister64(handle, reg, value));
            else
                CheckError(BlWriteRegister86(handle, reg, value));
        }

        [DllImport(DLLx86, EntryPoint = "BlReadBlock", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlReadBlock86(int handle, uint addr, byte[] buffer, int size, int flags);
        [DllImport(DLLx64, EntryPoint = "BlReadBlock", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlReadBlock64(int handle, uint addr, byte[] buffer, int size, int flags);
        [DllImport(DLLNoA, EntryPoint = "BlReadBlock", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlReadBlockNA(int handle, uint addr, byte[] buffer, int size, int flags);
        internal static void BlReadBlock(int handle, uint reg, byte[] buffer, int flags)
        {
            if (IsNoArch)
                CheckError(BlReadBlockNA(handle, reg, buffer, buffer.Length, flags));
            else if (Is64)
                CheckError(BlReadBlock64(handle, reg, buffer, buffer.Length, flags));
            else
                CheckError(BlReadBlock86(handle, reg, buffer, buffer.Length, flags));
        }

        [DllImport(DLLx86, EntryPoint = "BlWriteBlock", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlWriteBlock86(int handle, uint addr, byte[] buffer, int size, int flags);
        [DllImport(DLLx64, EntryPoint = "BlWriteBlock", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlWriteBlock64(int handle, uint addr, byte[] buffer, int size, int flags);
        [DllImport(DLLNoA, EntryPoint = "BlWriteBlock", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlWriteBlockNA(int handle, uint addr, byte[] buffer, int size, int flags);
        internal static void BlWriteBlock(int handle, uint reg, byte[] buffer, int flags)
        {
            if (IsNoArch)
                CheckError(BlWriteBlock64(handle, reg, buffer, buffer.Length, flags));
            else if (Is64)
                CheckError(BlWriteBlock64(handle, reg, buffer, buffer.Length, flags));
            else
                CheckError(BlWriteBlock86(handle, reg, buffer, buffer.Length, flags));
        }

        /// FPGA related.
        [DllImport(DLLx86, EntryPoint = "BlResetFpga", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlResetFpga86(int handle);
        [DllImport(DLLx64, EntryPoint = "BlResetFpga", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlResetFpga64(int handle);
        [DllImport(DLLNoA, EntryPoint = "BlResetFpga", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlResetFpgaNA(int handle);
        internal static void BlResetFpga(int handle)
        {
            if (IsNoArch)
                CheckError(BlResetFpgaNA(handle));
            else if (Is64)
                CheckError(BlResetFpga64(handle));
            else
                CheckError(BlResetFpga86(handle));
        }

        [DllImport(DLLx86, EntryPoint = "BlProgramFpgaFromBin", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlProgramFpgaFromBin86(int handle, string fileName);
        [DllImport(DLLx64, EntryPoint = "BlProgramFpgaFromBin", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlProgramFpgaFromBin64(int handle, string fileName);
        [DllImport(DLLNoA, EntryPoint = "BlProgramFpgaFromBin", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlProgramFpgaFromBinNA(int handle, string fileName);
        internal static void BlProgramFpgaFromBin(int handle, string fileName)
        {
            if (IsNoArch)
                CheckError(BlProgramFpgaFromBinNA(handle, fileName));
            else if (Is64)
                CheckError(BlProgramFpgaFromBin64(handle, fileName));
            else
                CheckError(BlProgramFpgaFromBin86(handle, fileName));
        }

        [DllImport(DLLx86, EntryPoint = "BlProgramFpgaFromMemory", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlProgramFpgaFromMemory86(int handle, byte[] buffer, int size, int compressed);
        [DllImport(DLLx64, EntryPoint = "BlProgramFpgaFromMemory", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlProgramFpgaFromMemory64(int handle, byte[] buffer, int size, int compressed);
        [DllImport(DLLNoA, EntryPoint = "BlProgramFpgaFromMemory", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlProgramFpgaFromMemoryNA(int handle, byte[] buffer, int size, int compressed);
        internal static void BlProgramFpgaFromMemory(int handle, byte[] buffer, bool compressed)
        {
            if (IsNoArch)
                CheckError(BlProgramFpgaFromMemoryNA(handle, buffer, buffer.Length, compressed ? 1 : 0));
            else if (Is64)
                CheckError(BlProgramFpgaFromMemory64(handle, buffer, buffer.Length, compressed ? 1 : 0));
            else
                CheckError(BlProgramFpgaFromMemory86(handle, buffer, buffer.Length, compressed ? 1 : 0));
        }

        /// Misc.
        [DllImport(DLLx86, EntryPoint = "BlSetTimeout", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetTimeout86(int handle, int timeout);
        [DllImport(DLLx64, EntryPoint = "BlSetTimeout", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetTimeout64(int handle, int timeout);
        [DllImport(DLLNoA, EntryPoint = "BlSetTimeout", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetTimeoutNA(int handle, int timeout);
        internal static void BlSetTimeout(int handle, int timeout)
        {
            if (IsNoArch)
                CheckError(BlSetTimeoutNA(handle, timeout));
            else if (Is64)
                CheckError(BlSetTimeout64(handle, timeout));
            else
                CheckError(BlSetTimeout86(handle, timeout));
        }

        [DllImport(DLLx86, EntryPoint = "BlSetUserId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetUserId86(int handle, StringBuilder userId);
        [DllImport(DLLx64, EntryPoint = "BlSetUserId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetUserId64(int handle, StringBuilder userId);
        [DllImport(DLLNoA, EntryPoint = "BlSetUserId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlSetUserIdNA(int handle, StringBuilder userId);
        internal static void BlSetUserId(int handle, string userId)
        {
            var r = new StringBuilder(BL_IF_BUFFER_SIZE);
            r.Append(userId);
            if (IsNoArch)
                CheckError(BlSetUserIdNA(handle, r));
            else if (Is64)
                CheckError(BlSetUserId64(handle, r));
            else
                CheckError(BlSetUserId86(handle, r));
        }

        [DllImport(DLLx86, EntryPoint = "BlGetUserId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetUserId86(int handle, StringBuilder buffer);
        [DllImport(DLLx64, EntryPoint = "BlGetUserId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetUserId64(int handle, StringBuilder buffer);
        [DllImport(DLLNoA, EntryPoint = "BlGetUserId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetUserIdNA(int handle, StringBuilder buffer);
        internal static string BlGetUserId(int handle)
        {
            var r = new StringBuilder(BL_IF_BUFFER_SIZE);
            if (IsNoArch)
                CheckError(BlGetUserIdNA(handle, r));
            else if (Is64)
                CheckError(BlGetUserId64(handle, r));
            else
                CheckError(BlGetUserId86(handle, r));
            return r.ToString();
        }

        [DllImport(DLLx86, EntryPoint = "BlGetSerialNumber", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetSerialNumber86(int handle, StringBuilder buffer);
        [DllImport(DLLx64, EntryPoint = "BlGetSerialNumber", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetSerialNumber64(int handle, StringBuilder buffer);
        [DllImport(DLLNoA, EntryPoint = "BlGetSerialNumber", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetSerialNumberNA(int handle, StringBuilder buffer);
        internal static string BlGetSerialNumber(int handle)
        {
            var r = new StringBuilder(BL_IF_BUFFER_SIZE);
            if (IsNoArch)
                CheckError(BlGetSerialNumberNA(handle, r));
            else if (Is64)
                CheckError(BlGetSerialNumber64(handle, r));
            else
                CheckError(BlGetSerialNumber86(handle, r));
            return r.ToString();
        }

        [DllImport(DLLx86, EntryPoint = "BlGetFirmwareVersion", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetFirmwareVersion86(int handle, ref int major, ref int minor, ref int fix);
        [DllImport(DLLx64, EntryPoint = "BlGetFirmwareVersion", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetFirmwareVersion64(int handle, ref int major, ref int minor, ref int fix);
        [DllImport(DLLNoA, EntryPoint = "BlGetFirmwareVersion", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetFirmwareVersionNA(int handle, ref int major, ref int minor, ref int fix);
        internal static void BlGetFirmwareVersion(int handle, ref int major, ref int minor, ref int fix)
        {
            if (IsNoArch)
                CheckError(BlGetFirmwareVersionNA(handle, ref major, ref minor, ref fix));
            else if (Is64)
                CheckError(BlGetFirmwareVersion64(handle, ref major, ref minor, ref fix));
            else
                CheckError(BlGetFirmwareVersion86(handle, ref major, ref minor, ref fix));
        }

        [DllImport(DLLx86, EntryPoint = "BlGetDerivateInfo", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetDerivateInfo86(int handle, StringBuilder buffer);
        [DllImport(DLLx64, EntryPoint = "BlGetDerivateInfo", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetDerivateInfo64(int handle, StringBuilder buffer);
        [DllImport(DLLNoA, EntryPoint = "BlGetDerivateInfo", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetDerivateInfoNA(int handle, StringBuilder buffer);
        internal static string BlGetDerivateInfo(int handle)
        {
            var r = new StringBuilder(BL_IF_BUFFER_SIZE);
            if (IsNoArch)
                CheckError(BlGetDerivateInfoNA(handle, r));
            else if (Is64)
                CheckError(BlGetDerivateInfo64(handle, r));
            else
                CheckError(BlGetDerivateInfo86(handle, r));
            return r.ToString();
        }

        [DllImport(DLLx86, EntryPoint = "BlGetDerivateId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetDerivateId86(int handle, ref uint id);
        [DllImport(DLLx64, EntryPoint = "BlGetDerivateId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetDerivateId64(int handle, ref uint id);
        [DllImport(DLLNoA, EntryPoint = "BlGetDerivateId", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        private static extern int BlGetDerivateIdNA(int handle, ref uint id);
        internal static uint BlGetDerivateId(int handle)
        {
            uint r = 0;
            if (IsNoArch)
                CheckError(BlGetDerivateIdNA(handle, ref r));
            else if (Is64)
                CheckError(BlGetDerivateId64(handle, ref r));
            else
                CheckError(BlGetDerivateId86(handle, ref r));
            return r;
        }
    }
}
