﻿/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/

// This example demonstrates most features of the BeastLink .NET API, coded in C#.

using System;
using System.IO;
using BeastLink;
using System.Linq;

namespace example
{
    class Example
    {
        // We use VID and PID for EFM-03 devices.
        const int VID = 0x10f8;
        const int PID = 0xc583;

        // Do a device/FPGA design example specific test for each
        // device that ships with an example design.
        private void DeviceTest()
        {
            // Loop over all devices.
            foreach (var ed in LibraryInterface.Enumerate(VID, PID))
            {
                // Open the device.
                using (var d = ed.Open()) 
                {
                    // This is the example design by Cesys, the design is read from 
                    // the specified file and programmed to the FPGA on the device (must be .bin format).
                    // Optionally, ProgramFpgaFromMemory() can
                    // be used to download a compiled in resource (generated by tools).
                    // ResetFpga() is explicitly invoked from within this call.
                    d.ProgramFpgaFromBin("./beastlink_exdes_top.bin");

                    // The example maps 16K Block RAM onto the AXI-4 Bus, starting at address 0x00000000.
                    const uint BlockRAMAddr = 0x00000000;
                    const int BlockRAMSize = 16 * 1024;
                    const int HalfRAMSize = BlockRAMSize / 2;

                    // Create 16kb block containing counting 32 bit values.
                    var data = Enumerable
                        .Range(0, BlockRAMSize / 4)
                        .SelectMany(BitConverter.GetBytes)
                        .ToArray()
                        ;

                    // Write to Block RAM.
                    d.WriteBlock(BlockRAMAddr, data);

                    // Read back, starting at 8k.
                    var rbdata = d.ReadBlock(BlockRAMAddr + HalfRAMSize, HalfRAMSize);

                    // Compare second half of written data to read back data.
                    var shalf = data.ToList().GetRange(HalfRAMSize, HalfRAMSize).ToArray();

                    // Write compare result.
                    Console.WriteLine("Sequences are " + (rbdata.SequenceEqual(shalf) ? "equal" : "not equal") + ".");
                }
            }
        }

        /// <summary>
        /// Enumerate all known devices and write information to stdout.
        /// </summary>
        private void InfoTest()
        {
            // Loop over all devices.
            foreach (var ed in LibraryInterface.Enumerate(VID, PID))
            {
                // EnumeratedDevice.Open() returns the opened instance to the device.
                using (var d = ed.Open()) 
                {
                    // This structure contains all information about the device.
                    // Not all devices have identical information. Devices, which
                    // don't have a property return a default value, no error is thrown.
                    var i = d.DeviceInfo;

                    // Write all informations to stdout, values are self explaining.
                    Console.WriteLine("Device found:");
                    Console.WriteLine("  Vid             : " + string.Format("{0:X4}", i.Vid));
                    Console.WriteLine("  Pid             : " + string.Format("{0:X4}", i.Pid));
                    Console.WriteLine("  UserId          : " + i.UserId);
                    Console.WriteLine("  SerialNumber    : " + i.SerialNumber);
                    Console.WriteLine("  FirmwareVersion : " + i.FirmwareVersion);
                    Console.WriteLine("  DerivateInfo    : " + i.DerivateInfo);
                    Console.WriteLine("  DerivateId      : 0x" + string.Format("{0:X8}", i.DerivateId));
                }
            }
        }

        /// <summary>
        /// Application entry point.
        /// </summary>
        /// <param name="args"></param>
        private void Run()
        {
            // Any error is reported by IOException, catch it globally.
            try
            {
                // Initialize API before using it, this loads the individual
                // modules that must be located in the working directory.
                LibraryInterface.Init();

                // Output UDK3 version to stdout.
                Console.WriteLine("BeastLink version : " + LibraryInterface.Version);

                // Set log level to log up to warning level (default).
                // Valid log levels are:
                // - 0x00: Off
                // - 0x01: Error
                // - 0x02: Warning
                // - 0x03: Debug
                LibraryInterface.SetLogLevel(0x2);

                // Write all informations about all known connected
                // devices to stdout.
                InfoTest();

                // Do a device specific test (with devices that have an 
                // example FPGA design).
                DeviceTest();

                // Cleanup internal structures. This closes all
                // opened resources, no further use of the UDK3 API
                // is possible until Init()is called again.
                LibraryInterface.Cleanup();
            }
            catch (IOException ex)
            {
                Console.WriteLine("Error: " + ex);
            }
        }

        static void Main(string[] args)
        {
            new Example().Run();
        }
    }
}
