#!/usr/bin/env python
# CESYS Software License - Version 1.0 - January 1st, 2016
#
# Permission is hereby granted, free of charge, to any person or organization
# obtaining a copy of the software and accompanying documentation covered by
# this license (the "Software") to use, reproduce, display, distribute,
# execute, and transmit the Software, and to prepare derivative works of the
# Software, and to permit third-parties to whom the Software is furnished to
# do so, all subject to the following:
#
# The software must only be used to operate hardware manufactured by
# CESYS GmbH, Herzogenaurach.
#
# The copyright notices in the Software and this entire statement, including
# the above license grant, this restriction and the following disclaimer,
# must be included in all copies of the Software, in whole or in part, and
# all derivative works of the Software, unless such copies or derivative
# works are solely in the form of machine-executable object code generated by
# a source language processor.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
# SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
# FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
#
# Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018

# This example demonstrates most features of the BeastLink Python API, compatible to
# Python 2.x and 3.x.

import beastlink, sys, array, time

# We use VID and PID for EFM-03 devices.
VID = 0x10f8
PID = 0xc583

def deviceTest():
    """
    Do a device/FPGA design example specific test for each
    device that ships with an example design.
    """
    # Loop over all devices.
    for ed in beastlink.enumerate(VID, PID):
        # Open the device.
        with ed.open() as d:
            # This is the example design by Cesys, the design is read from
            # the specified file and programmed to the FPGA on the device (must be .bin format).
            # Optionally, programFpgaFromMemory() can
            # be used to download a compiled in resource (generated by tools).
            # resetFpga() is explicitly invoked from within this call.
            d.programFpgaFromBin("./beastlink_exdes_top.bin")

            # The example maps 16K Block RAM onto the AXI-4 Bus, starting at address 0x00000000.
            BlockRAMAddr = 0x00000000
            BlockRAMSize = 16 * 1024
            HalfRAMSize = int(BlockRAMSize / 2)

            # Create 16kb block containing counting 32 bit values.
            data = array.array("I", [ i for i in range(int(BlockRAMSize / 4)) ])

            # Write to Block RAM.
            d.writeBlock(BlockRAMAddr, data)

            # Read back, starting at 8k.
            rbdata = array.array("I", [ 0 for i in range(int(HalfRAMSize / 4))])
            d.readBlock(BlockRAMAddr + HalfRAMSize, rbdata)

            # Compare second half of written data to read back data.
            shalf = array.array('I', data.tolist()[int(HalfRAMSize / 4):])

            # Write compare result.
            print("Sequences are %s." % ("equal" if rbdata == shalf else "not equal"))

def infoTest():
    """
    Enumerate all known devices and write information to stdout.
    """
    # Loop over all devices.
    for ed in beastlink.enumerate(VID, PID):
        # EnumeratedDevice.open() returns the opened instance to
        # the device.
        with ed.open() as d:
            # This structure contains all information about the device.
            # Not all devices have identical information. Devices, which
            # don't have a property return a default value, no error is thrown.
            i = d.getDeviceInfo()

            # Write all information to stdout, values are self explaining.
            print("Device found:")
            print("  Vid             : %04x" % i.getVid())
            print("  Pid             : %04x" % i.getPid())
            print("  UserId          : %s" % i.getUserId())
            print("  SerialNumber    : %s" % i.getSerialNumber())
            print("  FirmwareVersion : %i.%i.%i" % i.getFirmwareVersion())
            print("  DerivateInfo    : %s" % i.getDerivateInfo())
            print("  DerivateId      : 0x%08x" % i.getDerivateId())

def main():
    """
    Application entry point.
    """
    # Any error is reported by Exception, catch it globally.
    # As Python dumps exceptions by default, we don't handle
    # them here explicitly !

    # Initialize API before using it, this loads the individual
    # modules that must be located in the working directory.
    beastlink.init()

    # Output UDK3 version to stdout.
    print("BeastLink Version : %i.%i.%i" % beastlink.getVersion())

    # Set log level to log up to warning level (default).
    # Valid log levels are:
    # - 0x00: Off
    # - 0x01: Error
    # - 0x02: Warning
    # - 0x03: Debug
    beastlink.setLogLevel(0x2)

    # Write all information about all known connected
    # devices to stdout.
    infoTest()

    # Do a device specific test (with devices that have an
    # example FPGA design).
    deviceTest()

    # Cleanup internal structures. This closes all
    # opened resources, no further use of the UDK3 API
    # is possible until init() is called again.
    beastlink.cleanup()

if __name__ == '__main__':
    main()
