/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

package com.cesys.beastlink;

import java.io.IOException;
import static com.cesys.beastlink.SystemConnector.*;

/**
 * This type offers access to most of the device information. Some information
 is available after opening the device. An instance to this type can either
 * be retrieved from EnumeratedDevice.getDeviceInfo() or Device.getDeviceInfo().
 */
public class DeviceInfo {
	private int _handle = BL_INVALID_DEVICE;
	private final int _vid, _pid;

	void setHandle(int handle) {
		_handle = handle;
	}

	/**
	 * Return vendor ID of device (if applicable).
	 */
	public int getVid() {
		return _vid;
	}

	/**
	 * Return product ID of device (if applicable).
	 */
	public int getPid() {
		return _pid;
	}

	/**
	 * Set user ID. Writes the string to the device EEPROM.
	 * This ID is intended to differentiate multiple devices of the same type connected
	 * to one host. The ID can contain up to 63 ASCII characters.
	 * @param userId ID to write.
	 */
	public void setUserId(String userId) throws IOException {
		if (_handle == BL_INVALID_DEVICE)
			throw new IOException("Device not opened.");

		byte[] r = new byte[BL_IF_BUFFER_SIZE];
		System.arraycopy(userId.getBytes(), 0, r, 0, userId.getBytes().length);
		checkError(BlSetUserId(_handle, r));
	}

	/**
	 * Return user ID from device EEPROM. See explanation in setUserId().
	 * @return User id, "" if not accessible or in case of error.
	 */
	public String getUserId() throws IOException {
		if (_handle == BL_INVALID_DEVICE)
			return "";

		try {
			byte[] r = new byte[BL_IF_BUFFER_SIZE];
			checkError(BlGetUserId(_handle, r));
			return convertByteArrayToString(r);
		} catch (IOException ex) {
			return "";
		}
	}

	/**
	 * Return serial number. Not supported by all device types / instances.
	 * @return Serial number, "" if not accessible or in case of error.
	 */
	public String getSerialNumber() throws IOException {
		if (_handle == BL_INVALID_DEVICE)
			return "";

		try {
			byte[] r = new byte[BL_IF_BUFFER_SIZE];
			checkError(BlGetSerialNumber(_handle, r));
			return convertByteArrayToString(r);
		} catch (IOException ex) {
			return "";
		}
	}

	/**
	 * Return firmware version of device.
	 * @return Firmware version, 0.0.0 in case of error.
	 */
	public Version getFirmwareVersion() throws IOException {
		if (_handle == BL_INVALID_DEVICE)
			return new Version(0, 0, 0);

		try {
			int[] major = new int[1], minor = new int[1], fix = new int[1];
			checkError(BlGetFirmwareVersion(_handle, major, minor, fix));
			return new Version(major[0], minor[0], fix[0]);
		} catch (IOException ex) {
			return new Version(0, 0, 0);
		}
	}

	/**
	 * Return derivate info string.
	 * Some devices are available in different versions (different FPGA's, different peripherals).
	 * The returned string contains a textual information about the specific
	 * derivate. For programmatically differentiation, getDerivateId() is better suited.
	 * @return Derivate info string, "" if not accessible or in case of error.
	 */
	public String getDerivateInfo() throws IOException {
		if (_handle == BL_INVALID_DEVICE)
			return "";

		try {
			byte[] r = new byte[BL_IF_BUFFER_SIZE];
			checkError(BlGetDerivateInfo(_handle, r));
			return convertByteArrayToString(r);
		} catch (IOException ex) {
			return "";
		}
	}

	/**
	 * Return derivate id.
	 * This ID is unique for every derivate of a specific device type.
	 * Existing derivates can be found in the documentation of the specific device type.
	 * @return Derivate id, 0xffffffff if not accessible or in case of error.
	 * @throws IOException
	 */
	public long getDerivateId() throws IOException {
		if (_handle == BL_INVALID_DEVICE)
			return 0xffffffffL;

		try {
			int[] r = new int[1];
			checkError(BlGetDerivateId(_handle, r));
			return fromUInt32(r[0]);
		} catch (IOException ex) {
			return 0xffffffffL;
		}
	}

	DeviceInfo(int vid, int pid) {
		_vid = vid;
		_pid = pid;
	}
}
