/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

package com.cesys.beastlink;

import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static com.cesys.beastlink.SystemConnector.*;

/**
 * Class that is responsible for all device independent functionality.
 */
public class LibraryInterface {
	static int _enumRound = 0;

	/**
	 * Return full version of API.
	 * @return BeastLink version.
	 */
	public static Version getVersion() {
		int[] major = new int[1], minor = new int[1], fix = new int[1];
		BlGetVersion(major, minor, fix);
		return new Version(major[0], minor[0], fix[0]);
	}

	/**
	 * The API is not usable before this function is called.
	 * @throws IOException
	 */
	public static void init() throws IOException {
		checkError(BlInit());
	}

	/**
	 * Search for devices masked by the given Vendor and Product ID (VID, PID).
	 * @param type Filter search by class or type.
	 * @return List of detected devices. Instances are valid until the next call to enumerate.
	 * @throws IOException
	 */
	public static List<EnumeratedDevice> enumerate(int vid, int pid) throws IOException {
		++_enumRound;

		int[] count = new int[1];
		checkError(BlEnumerate(vid, pid, count));

		// Transform to list and return result.
		return IntStream
				.range(0, count[0])
				.mapToObj(x -> new EnumeratedDevice(new DeviceInfo(vid, pid), x, _enumRound))
				.collect(Collectors.toList())
				;
	}

	/**
	 * Free API resources. To reuse the API again, init() must be called again.
	 */
	public static void cleanup() {
		try {
			checkError(BlCleanup());
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Set logging level (Off = 0, Error = 1, Warning = 2, Debug = 3), default is 2.
     * @param level
	 * @throws IOException
	 */
	public static void setLogLevel(int level) throws IOException {
		checkError(BlSetLogLevel(level));
	}
}
